%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                        Model B                            %
%                Kyasha, Piotr and Anita                    %
%                    20th August 2010                       %
% Program simulating designed 3-step amplification up to    %
% dioxygenase output which includes enzymatic reaction of   %
% TEV on split TEV and inactive dioxygenase.                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function p_prime = model_b(time,p)

%% Initial Reactants
%T + sTa + sTb + sD

%% Abbreviations
%p(1) = [T] = [TEV]e
%p(2) = [sD] = [splitDioxygenase]
%p(3) = [T-sD] = [TEV-splitDioxygenase]
%p(4) = [D] = [Dioxygenase]
%p(5) = [sTa] = [splitTEV(a)]
%p(6) = [sTb] = [splitTEV(b)]
%p(7) = [Tsa] = [activated TEV(a)]
%p(8) = [Tsb] = [activated TEV(b)]
%p(9) = [Ts] = [activated TEV]
%p(10) = [T-sTa] = [TEV-splitTEV(a)]
%p(11) = [T-sTb] = [TEV-splitTEV(b)]
%p(12) = [Ts-sD] = [transition state Ts-sD]
%p(13) = [Tsa-Tsb] = [activated TEV(a)-activated TEV(b)]
%% Units
% [p_dprime] = [mol*dm^-3*s^-1]
% [s_d] = [mol*dm^-3*s^-1]
% [d_d] = [s^-1]
% [p_d] = [mol*dm^-3]

%% Declaring global variables
global s_sd %production rate constant of splitDioxygenase
global s_t %production rate constant of TEV
global deg_c %common degradation constant due to cell division
global k_TEV %k_cat value of TEV protease acting on the substrate

%% Production and Degradation constants
s_sTa = s_t/1.889;  %production rate constant of sTa
s_sTb = s_t/1.889;  %production rate constant of sTb

d_t = deg_c;    %degradation rate constant of T
d_sTa = deg_c;  %degradation rate constant of sTa
d_sTb = deg_c;  %degradation rate constant of sTb
d_Tsa = deg_c;  %degradation rate constant od Tsa
d_Tsb = deg_c;  %degradation rate constant od Tsb
d_sd = deg_c;   %degradation rate constant of sD
d_ts = deg_c;   %degradation rate cpnstant of Ts 
d_d = deg_c;    %degradation rate constant of D

%d_TssD = 0; %degradation rate constant od Ts-sD neglected
%d_TsTb = 0; %degradation rate constant od T-sTb neglected
%d_TsTa = 0; %degradation rate constant od T-sTa neglected
%% Kinetic rate constants

% 1) T+sTa+sTb+sD <-> T-sD -> T+D+sTa+sTb
k011 = 1e7; % T + sD -> T-sD 
k012 = 610; % T + sD <- T-sD
k013 = k_TEV; % T-sD -> T + D
% 2) T + sTa + sTb + sD -- T-sTa --> T + Tsa + sTb + sD
k021 = 1e7; % T + sTa -> T-sTa
k022 = 610; % T + sTa <- T-sTa
k023 = k_TEV; % T-sTa -> T + Tsa
% 3) T + Tsa + sTb + D --  T-sTb --> T + Tsa + Tsb + D
k031 = 1e7; % T + sTb -> T-sTb
k032 = 610; % T + sTb <- T-sTb
k033 = k_TEV; % T-sTb -> T + Tsb
% 4) Ts + T + sD -- Ts-sD --> Ts + T + D
k041 = 0.4*k011; % Ts + sD -> Ts-sD
k042 = 1.4*k012; % Ts + sD <- Ts-sD
k043 = 0.4*k_TEV; % Ts-sD -> Ts + D
% 5) T + Tsa + Tsb + D --  Tsa-Tsb --> T + Ts + D
k051 = 100; % Tsa + Tsb -> Ts  % !! This is made up !!
k052 = 0.1; % Tsa + Tsb <- Ts  % !! This is made up !!

p_prime = zeros(12,1);
%% p_prime(1) - TEV production rate
% Terms coming from reactions involving: T-sTa T-sD T-sTb
p_prime(1) = -k011*p(1)*p(2) + (k012+k013)*p(3)-k021*p(1)*p(5) + (k022+k023)*p(10) -k031*p(1)*p(6) + (k032+k033)*p(11)+ s_t - d_t*p(1);

%% p_prime(2) - Production rate of splitDioxygenase
% Terms coming from reactions involving: T-sD,Ts-sD
p_prime(2) = -k011*p(1)*p(2) + k012*p(3)-k041*p(9)*p(2) + k042*p(12)+ s_sd - d_sd*p(2);

%% p_prime(3) - Production rate of TEV-splitDioxygenase 
% Terms coming from reactions involving: T-sD
p_prime(3) = k011*p(1)*p(2) - (k012+k013)*p(3); % Negligible degradation term for transition state

%% p_prime(4) - Production rate of Dioxygenase
% Terms coming from reactions involving: T-sD Ts-sD
p_prime(4) = k013*p(3)+ k043*p(12) - d_d*p(4);

%% p_prime(5) - Production rate of sTa
% Terms coming from reactions involving: T-sTa
p_prime(5) = -k021*p(1)*p(5) + k022*p(10) + s_sTa - d_sTa*p(5);

%% p_prime(6) - Production rate of sTb
% Terms coming from reactions involving: T-sTb
p_prime(6) = -k031*p(1)*p(6) + k032*p(11) + s_sTb - d_sTb*p(6);

%% p_prime(7) - Production rate of Tsa
% Terms coming from reactions involving: T-sTa Tsa-Tsb
p_prime(7) = k023*p(10)-k051*p(7)*p(8) + k052*p(9) - d_Tsa*p(7); 

%% p_prime(8) - Production rate of Tsb
% Terms coming from reactions involving: T-sTb Tsa-Tsb
p_prime(8) = k033*p(11) -k051*p(7)*p(8) + k052*p(9)- d_Tsb*p(8); 

%% p_prime(9) - Production rate of activated TEV [Ts]
% Terms coming from reactions involving: Tsa-Tsb
p_prime(9) = k051*p(7)*p(8) - k052*p(9) -k041*p(9)*p(2) + (k042+k043)*p(12) - d_ts*p(9);

%% p_prime(10) - Production rate of TEV-splitTEV(a) [T-sTa]
% Terms coming from reactions involving:  T-sTa
p_prime(10) = k021*p(1)*p(5) - (k022+k023)*p(10); % Negligible degradation term for transition state

%% p_prime(11) - Production rate of TEV-splitTEV(b) [T-sTb]
% Terms coming from reactions involving: T-sTb
p_prime(11) = k031*p(1)*p(6) - (k032+k033)*p(11); % Negligible degradation term for transition state

%% p_prime(12) - Production rate of transition state Ts-sD[Ts-sD]
% Terms coming from reactions involving: Ts-sD
p_prime(12) = k041*p(9)*p(2) - (k042+k043)*p(12); % Negligible degradation term for transition state

%% p_prime(13) - Production rate of activated TEV(a)-activated TEV(b) [Tsa-Tsb]
% This reaction is not ensymatic -> This Ttransition State doers not exist

end