%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                      Run Output                            %
%                    Anita and Piotr                         %
%                  03th September 2010                       %
% Program simulating colour response (catechol-> colour).    %
%                                                            %
% Please, read the attached readme file before using.        %
%                                                            %
% Depending on choice of parameter values, especially the    %
% ones that are declared global the model's behaviour will   %
% change.                                                    %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Declare global variables
%Global variables are used to pass on values of variables into other functions.

global s_sd %production rate constant of splitDioxygenase
global s_t %production rate constant of TEV
global deg_c %common degradation constant due to cell division
global k_TEV %k_cat value of TEV protease acting on the substrate
global k_d %k_cat value of dioxygenase acting on the substrate

%% Define initial constants

c0=1e-5;                % [mol*dm^-3] - initial concentration of split dioxygenase
c_catechol = 9e-5;      % [mol*dm^-3] - initial concentration of Catechol
t_stop=1;               % [s] (choose values between 1 and 2000) time when catechol is added to cells after setting of outputs 
s_sd = 1*10^(-11);      % [mol*dm^-3*s^-1] production rate constant of splitDioxygenase
s_t = 2.3808*10^(-10);  % [mol*dm^-3*s^-1] production rate constant of TEV
deg_c = 0.000289;       % [s^-1] common degradation constant due to cell division
k_TEV = 0.16;           % [s^-1] k_cat value of TEV protease acting on the substrate
k_d = 52;               % [s^-1] k_cat value of dioxygenase acting on the substrate

%% Run 3 ODEs of 3 different amplification models without catechol 
% You can choose when to add catechol

step=1; % [s] - size of the time step for evaluation of systems before catechol is added.
time = 0:step:2000; % [s] % time vector of ODEs in this cell

%Model preA
[time,preA] = ode15s(@model_pre_a,time,0);

% Model A
options1 = odeset('NonNegative',[1 1 1 1]);
[time,modA] = ode15s(@model_a,time, [0 c0 0 0], options1);

%Model B
options2 = odeset('NonNegative',[1 1 1 1 1 1 1 1 1 1 1 1]);
[time,modB] = ode15s(@model_b,time,[0 c0 0 0 0 0 0 0 0 0 0 0], options2);

%% Cell death simulation
% If you wish to simulate the response as if catechol was instantenously
% killing cells uncomment the two lines below. They will set any production
% of proteins to 0 when catechol is added to system. Only already existing
% enzymes in the system will be allowed to continue functioning.

% s_sd = 0;
% s_t = 0;

%% Retrieve data from models at chosen time
t = t_stop/step;    % [s]  - Number in time array that t_stop corresponds to

%% Refine sensitvity of Matlab
rel_tol=c_catechol/1e15;%relative tolerance should not be too low nor too 
% high with respect to c_ctechol (this rule (c_catechol/1e15) fails for
% numbers higher than 1e1)
abs_tol=1e-10; %allows accurate evaluation, but needs to be made bigger eg.1e-5
% for bigger initial concentrations of cetechol.
%% Run the ODE for catechol for each of the models
%Define inital and final time base on the t_stop
t2_0=t_stop; % [s]
t2_f=t_stop+8e2; % [s] For how long the simulation should be continued after adding catechol to cells

%Options allow ode15s to evaluate with higher accuracy and hence prevent
%abnormal solutions due to quick changes in concentration of catechol.
options3 = odeset('RelTol',rel_tol,'NonNegative',[1 1 1 1],'AbsTol',abs_tol);
%colour response for model preA
[time1,color_preA] = ode15s(@catechol, [t2_0 t2_f], [preA(t)/4 c_catechol 0 0], options3);

options4 = odeset('RelTol',rel_tol,'NonNegative',[1 1 1 1 1 1 1],'AbsTol',abs_tol);
%colour response for model A
[time2,color_modA] = ode15s(@catechol_A, [t2_0 t2_f], [modA(t,1) modA(t,2) modA(t,3) modA(t,4)/4 c_catechol 0 0], options4);

options5 = odeset('RelTol',rel_tol,'NonNegative',[1 1 1 1 1 1 1 1 1 1 1 1 1 1 1],'AbsTol',abs_tol);
%colour response for model B
[time3,color_modB] = ode15s(@catechol_B, [t2_0 t2_f], [modA(t,1) modB(t,2) modB(t,3) modB(t,4)/4 modB(t,5) modB(t,6) modB(t,7) modB(t,8) modB(t,9) modB(t,10) modB(t,11) modB(t,12) c_catechol 0 0], options3);

%Convert time into minutes
time=time/60;
time1=time1/60; 
time2=time2/60;
time3=time3/60;

%Convert units from M to uM
color_preA=color_preA*1e6;
color_modA=color_modA*1e6;
color_modB=color_modB*1e6;


%% Plot
figure
subplot(2,2,1), plot(time1,color_preA(:,1))
title('Production of C230 (Dioxygenase)')
xlabel('time [min]')
ylabel('concentration [uM]')
hold on
plot(time3,color_modB(:,4),'r')
plot(time2,color_modA(:,4),'g')
hold off

subplot(2,2,2), plot(time1,color_preA(:,2))
title('Production of Catechol')
xlabel('time [min]')
ylabel('concentration [uM]')
hold on
plot(time3,color_modB(:,13),'r')
plot(time2,color_modA(:,5),'g')
hold off

subplot(2,2,3), plot(time1,color_preA(:,3))
title('Production of intermediate complex')
xlabel('time [min]')
ylabel('concentration [uM]')
hold on
plot(time3,color_modB(:,14),'r')
plot(time2,color_modA(:,6),'g')
hold off

subplot(2,2,4), 
plot(time1,color_preA(:,4))
title('Production of coloured compound - Muconic acid')
xlabel('time [min]')
ylabel('concentration [uM]')
hold on
plot(time3,color_modB(:,15),'r')
plot(time2,color_modA(:,7),'g')
% Draw a line of the visibility threshold
thres=zeros(numel(time1),1);
thres(:,1)=40; %[uM]
plot(time1,thres,'k') %Plot the threshold value
hold off